XenoProbes
==========

This program is a utility for the game **Xenoblade Chronicles X**, for the Wii
U. In the game, the player is required to install data probes that extract
resources from certain sites. There are various elements that change the output
of resources, so I wrote this program to automatically search for the optimal
configuration.


Usage (GUI)
-----------

There's a simple graphical interface to the program, "xenogui". There are 4
main groups of configurations:

 - Priorities: choose what aspect (storage, revenue or miranium production)
   will be maximized first.

 - Options: affect how the optimizer is going to work. The defaults should
   work well enough; higher numbers make it take longer and/or use more memory,
   but might give more optimized results. See the documentation for the
   command-line version below for details on each option.

 - Inventory: input how many of each probe you have.

 - Edit Sites: if you have not activated all probe sites, or have not found
   all sightseeing spots, you can update the state of your map here. Don't
   forget to hit *Save* afterwards (it's saved in *inventory.txt*.) Press ESC
   to close the window.

When done configuring (and the inventory is properly saved), click "Save
inventory and OPTIMIZE!" to start the optimization. If you see a black terminal
window, don't close it (that's where the real program is running.) On MS
Windows you can press "CTRL+C" on the black terminal window to stop the
optimization early (and use whatever partial result is the best.) Unix systems
can use the "STOP!" button instead.


Usage (command line version)
----------------------------

1. Copy the *sample-inventory.txt* file into *inventory.txt*. Open it up on a
   text editor, and change its contents to match your in-game inventory.

2. Did you discover all of Mira? Add extra lines to *inventory.txt*:

    - A line containing *-N* if you don't have a probe on site N yet, like
      this:

        `-101`
        
        `-102`

        `-502`
       
    - A line containing *+N,S* if you only have S *sightseeing* spots on site
     *N*, like this:
     
        `+214,1`

        `+513,0`


   up *sites.txt* in a text editor and update it to match your map. If
   you haven't discovered a site, delete (or comment out) the line. If you
   don't have all Sightseeing Spots for that site yet, change the number too.

3. In a command prompt/shell, from the top directory, run:

        ./xenoprobes

    or

        xenoprobes.exe

  And be patient. At any point you can press CTRL+C to make the program stop
  early. When it finishes, it'll print out the best configuration found, and
  the expected mining/revenue/storage total (minus some rounding errors.) Try
  running the program multiple times too, each time you might get a different
  (possibly better) solution.
  
  **Hint**: if you see the program getting stuck at some value, run another
  instance in another command prompt. If the new instance reaches a better value
  before the old one gets unstuck, kill the older one. That's the nature of
  randomness.


Program Details
---------------

Try:

    ./xenoprobes --help

or

    xenoprobes.exe --help

XenoProbes uses a Stochastic Hill Climbing method. It starts with multiple
randomly generated solutions; at every iteration, each solution is randomly
mutated into a number of candidate improvements, or "offsprings". If the best
offspring is better than the original solution, it replaces it; if not, the
solution is kept around for a while, to try again.

The options you should really try to tweak are:

* ``--storageweight``, ``--revenueweight`` and ``--productionweight``: those
  affect how the program finds solutions; by default, it gives priority to
  storage (*storageweight* = 1000), then revenue (*revenueweight* = 10), and
  lastly Miranium production (*productionweight* = 1). If you want revenue to
  be more important, change the weights so *revenueweight* is larger than the
  others. Most people will want them in this order though.

* ``--iterations``: you can make this really big, and leave it running
  overnight. Then come in the morning and just press CTRL+C to stop it.

* ``--population`` and ``--offsprings``: affect how many solutions are being
  tracked at any given time, and how many variations will be generated from
  each. The larger those numbers, the more likely you'll find a good solution,
  but it slows things down.

* ``--mutation``: it's the probability that a given probe will be swapped by
  another. This shouldn't be too high, or else whenever a mutation improves a
  solution, another one might happen that ruins it. It shouldn't be too low
  either, because sometimes many swaps are needed at once to improve a solution.

* ``--age``: at every iteration, if no offsprings are found to improve a solution,
  the original solution is kept in the pool, and starts aging. This parameter
  controls how old a solution can be before it's eliminated, and a fresh, random
  solution is generate in its place.

